//---------------------------------------------------------------------------
// jrdebug_core.h
// core function implementations
//---------------------------------------------------------------------------








//---------------------------------------------------------------------------
// constructor called by global singleton
JrDebug::JrDebug(bool doinit)
	{
	// this only does something if doinit==true, otherwise its just invoked to insure existence of global
	if (doinit)
	    {
		// default initializations
		singletonflag=true;
		debugindex=0;
		filtermin=-99999;
		displaypassingasserts=false;
		alarmfilterlevel=JrDebugIgnoreLevel;
		exceptionlevel=90;
		// misc
		jrcallbackp=NULL;
		// activity/debug blocks
		lastblockobjp=NULL;
		debugblockdepth=0;
		// set runid to current time
		time_t timenum;
		time(&timenum);
		runid=(long)timenum;
		// testobjects
		testobjects=NULL;
		lasttestobjp=NULL;
		curtestobjp=NULL;
		rantests=false;
		// formatters
		formatters=NULL;
		lastformatterp=NULL;
		formatter_factorys=NULL;
		lastformatter_factoryp=NULL;
		// filters
		filter_showallblocks=false;
		filter_onlyerrors=false;
		// startup enabled?
		#ifdef JrDebugDIRECTIVE_StartupDisabled
			enabled=false;
		#else
			enabled=true;
		#endif
		}
	else
	    {
		// this can get called by our singleton insurer class, in its efforts to auto register classes with the global debugger; do nothing
		}
	}


// constructor called by our clever macros
JrDebug::JrDebug(const char* infilename, int ilinenum , const char *intimestamp, const char *infunctionname, const char *infunctionname2, const char *infunctionname3, JrdType inmtype)
	{
	// constructor - DONT INITIALIZE STATICS HERE (see above)
	singletonflag=false;
	timestampstring = jrd_stdstringT(intimestamp);
	SetJrdType(inmtype);
	depth=debugblockdepth;
	activitymodstring = "";
	severitylevelnumber = JrDebugDefaultLevel;
	linenumber=ilinenum;
	shouldshowmessagebox=false;

	// function name, get the first one that matched
	if (infunctionname[0]!='\0')
		functionnamestring = jrd_stdstringT(infunctionname);
	else if (infunctionname2[0]!='\0')
		functionnamestring = jrd_stdstringT(infunctionname2);
	else
		functionnamestring = jrd_stdstringT(infunctionname3);

	// split filename into file and path
	filenamestring=jrd_stdstringT(infilename);
	size_t len=filenamestring.length();
	int count;
	if (len>0)
		{
		for (count=(int)(len-1);count>=0;--count)
			{
			if (filenamestring[count]=='\\' || filenamestring[count]=='/')
				break;
			}
		if (count>0)
			{
			filedirstring=filenamestring.substr(0,count+1);
			filenamestring=filenamestring.substr(count+1,len-(count+1));
			}
		else
			filedirstring="";
		}
	else
		filedirstring="";
	}

// destructor
JrDebug::~JrDebug()
	{
	// destructor - clean up any open file
	if (singletonflag)
		{
		// this is true only when global instance dies
		JrDebugRegisteredOutputFormatter* curp;
		JrDebugRegisteredOutputFormatter* thenextp;

		// pop any debug block stack
		PopAllDebugBlocks();

		// clean up registered test functions
		JrDebugRegisteredTestObj* rtop=testobjects;
		JrDebugRegisteredTestObj* rtopnext;
		while (rtop!=NULL)
			{
			rtopnext=rtop->getNextp();
			delete rtop;
			rtop=rtopnext;
			}

		// shutdown and remove any instantiated formatters
		RemoveInstantiatedFormatters();

		// clean up formatter factories
		curp=formatter_factorys;
		while (curp!=NULL)
			{
			thenextp=curp->get_nextp();
			delete curp;
			curp=thenextp;
			}

		// cleanup redirected win32console
		#ifdef _WIN32JRD
		 #ifndef JrDebugDIRECTIVE_DontFixWin32Console
			if (did_allocwin32console)
				FreeConsole();
			did_allocwin32console=false;
		 #endif
		#endif
		}
	}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void JrDebug::RemoveInstantiatedFormatters()
	{
	//
	JrDebugRegisteredOutputFormatter* curp;
	JrDebugRegisteredOutputFormatter* thenextp;

	// shut down all outputting formatters
	ShutdownFormatters();
	// clean up instantiated formatters
	curp=formatters;
	while (curp!=NULL)
		{
		thenextp=curp->get_nextp();
		delete curp;
		curp=thenextp;
		}
	formatters=NULL;
	}
//---------------------------------------------------------------------------








//---------------------------------------------------------------------------
// accessors
void JrDebug::SetFilterLevel(int min)
	{
	// set range of displayable messages
	filtermin=min;
	}
bool JrDebug::GetEnabled()
	{
	return enabled;
	}
void JrDebug::SetEnabled(bool val)
	{
	enabled=val;
	}
void JrDebug::SetEnabled(bool val,char *reasonstr,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3)
	{
	if (reasonstr!=NULL)
		{
		if (strcmp(reasonstr,"")!=0)
			{
			// announce that its being enabled or disabled
			enabled=true;
			if (val==true)
				{
				JrDebug( infilename, ilinenum, intimestamp, infunctionname , infunctionname2,infunctionname3, JrdMessage)(JrdNote,"","",JrDebugDefaultLevel,"JrDebug output enabled (%s)",reasonstr);
				}
			else
				{
				JrDebug( infilename, ilinenum, intimestamp, infunctionname , infunctionname2,infunctionname3, JrdMessage)(JrdNote,"","",JrDebugDefaultLevel,"JrDebug output disabled (%s)",reasonstr);
				}
			}
		}
	enabled=val;
	}
void JrDebug::SetDisplayPassingAsserts(bool val)
	{
	displaypassingasserts=val;
	}

void JrDebug::SetAlarmFilterLevel(int val)
	{
	alarmfilterlevel=val;
	}
void JrDebug::SetExceptionLevel(int val)
	{
	exceptionlevel=val;
	}
void JrDebug::DisableAlarmsFilter()
	{
	alarmfilterlevel=JrDebugIgnoreLevel;
	}
void JrDebug::DisableExceptionLogging()
	{
	exceptionlevel=JrDebugIgnoreLevel;
	}
void JrDebug::EnableExceptionLogging(int val)
	{
	exceptionlevel=val;
	}
void JrDebug::SetCallbackPointer(class JrDebugCallback *inp)
	{
	jrcallbackp=inp;
	}

// set message type string
void JrDebug::SetJrdType(const JrdType inmtype)
	{
	// ATTN: we should grab this from a static array instead of a switch
	messagetype=inmtype;
	switch(inmtype)
		{
		case JrdMessage:
			messagetypestring="Message";
			break;
		case JrdWarning:
			messagetypestring="Warning";
			break;
		case JrdError:
			messagetypestring="Error";
			break;
		case JrdAlarm:
			messagetypestring="Alarm";
			break;
		case JrdException:
			messagetypestring="Exception";
			break;
		case JrdNote:
			messagetypestring="Note";
			break;
		case JrdTest:
			messagetypestring="Test";
			break;
		case JrdCustom:
			messagetypestring="Custom";
			break;
		case JrdAssertPass:
			messagetypestring="AssertPass";
			break;
		case JrdAssertFail:
			messagetypestring="AssertFail";
			break;
		case JrdTestPass:
			messagetypestring="TestPass";
			break;
		case JrdTestFail:
			messagetypestring="TestFail";
			break;
		case JrdBlockStart:
			messagetypestring="BlockStart";
			break;
		case JrdBlockEnd:
			messagetypestring="BlockEnd";
			break;
		case JrdBlockEndError:
			messagetypestring="BlockEndErr";
			break;
		case JrdBlockInfo:
			messagetypestring="BlockInfo";
			break;
		case JrdBlockError:
			messagetypestring="BlockError";
			break;
		case JrdInternalInfo:
			messagetypestring="InternalInfo";
			break;
		case JrdInternalError:
			messagetypestring="InternalError";
			break;
		default:
			messagetypestring="Unknown";
			break;
		}
	}


bool JrDebug::MessageIsSimpleError()
	{
	// return a # hint which can aid coloring
	if (messagetype==JrdInternalError || messagetype==JrdBlockError || messagetype==JrdTestFail || messagetype==JrdException || messagetype==JrdError)
		return true;
	return false;
	}

bool JrDebug::MessageIsError()
	{
	// return a # hint which can aid coloring
	if (MessageIsSimpleError() || messagetype==JrdBlockEndError)
		return true;
	return false;
	}


void JrDebug::UpdateErrorCounts()
	{
	// update enclosing block if its an error
	if (MessageIsSimpleError())
		{
		// its an error type
		if (lastblockobjp!=NULL)
			lastblockobjp->IncError();
		}
	}
//---------------------------------------------------------------------------








































































//---------------------------------------------------------------------------
// invoke the formatter with printf style argument line
void JrDebug::operator()(const jrd_stdstringT str)
	{
	operator()("%s",str.c_str());
	}


void JrDebug::operator()(const char* format, ... )
	{
	// invoke formatter with vararg stuff
	va_list va;
	va_start(va, format);
	DoVa(va,format);
	}


// alternative argument line with special parameters OutputDebugPrintf(MessageType,SeverityNumber,...)
void JrDebug::operator()(const JrdType inmtype , const jrd_stdstringT str)
	{
	operator()(inmtype,str.c_str());
	}


void JrDebug::operator()(const JrdType inmtype , const char* format, ... )
	{
	// handle the case where we are passed mtype and severity
	SetJrdType(inmtype);
	// set severity level to default
	severitylevelnumber=JrDebugDefaultLevel;
	// messagebox?
	if (alarmfilterlevel!=JrDebugIgnoreLevel && severitylevelnumber>=alarmfilterlevel)
		shouldshowmessagebox=true;
	else
		shouldshowmessagebox=false;		

	// invoke formatter with varargstuff
	va_list va;
	va_start(va, format);
	DoVa(va,format);
	}


void JrDebug::operator()(const JrdType inmtype , const int inseverity, const jrd_stdstringT str)
	{
	operator()(inmtype,inseverity,"%s",str.c_str());
	}


void JrDebug::operator()(const JrdType inmtype , const int inseverity, const char* format, ... )
	{
	// handle the case where we are passed mtype and severity
	SetJrdType(inmtype);
	// set severity level
	severitylevelnumber=inseverity;

	// messagebox?
	if (alarmfilterlevel!=JrDebugIgnoreLevel && severitylevelnumber>=alarmfilterlevel)
		shouldshowmessagebox=true;
	else
		shouldshowmessagebox=false;		

	// invoke formatter with varargstuff
	va_list va;
	va_start(va, format);
	DoVa(va,format);
	}


// alternative argument line with special parameters OutputDebugPrintf(MessageType,SubTypeString,ActivityString,SeverityNumber,...)
void JrDebug::operator()(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int inseverity, const jrd_stdstringT str)
	{
	operator()(inmtype,dmsubtypestr,inactivity,inseverity,"%s",str.c_str());
	}


void JrDebug::operator()(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int inseverity, const char* format, ... )
	{
	// handle the case where we are passed mtype and severity
	SetJrdType(inmtype);
	// add custom subtype
	if (strcmp(dmsubtypestr,"")!=0)
		{
		if (messagetypestring=="Custom")
			{
			// use the custom type w/subtype to have type name appear alone
			messagetypestring=jrd_stdstringT(dmsubtypestr);
			}
		else
			{
			// append .subtype to type string
			messagetypestring+=(jrd_stdstringT(".")+jrd_stdstringT(dmsubtypestr));
			}
		}
	// activity mod
	SetActivityModString(inactivity);
	// set severity level
	severitylevelnumber=inseverity;

	// messagebox?
	if (inmtype==JrdAlarm || (alarmfilterlevel!=JrDebugIgnoreLevel && severitylevelnumber>=alarmfilterlevel))
		shouldshowmessagebox=true;
	else
		shouldshowmessagebox=false;		

	// invoke formatter with varargstuff
	va_list va;
	va_start(va, format);
	DoVa(va,format);
	}


// assert replacement
//void JrDebug::operator()(const bool assertnotverify, const bool assertcondition, const char *assertstring)
void JrDebug::operator()(const JrEnumDummyFunctionDisambiguator_Assert dummyvar , const bool assertnotverify, const bool assertcondition, const char *assertstring)
	{
	// assert string
	if (assertcondition)
		{
		if (displaypassingasserts)
			operator()(JrdAssertPass,"","",0,"%s",assertstring);
		}
	else
		operator()(JrdAssertFail,"","",100,"%s",assertstring);
	if (assertnotverify && !assertcondition)
		{
		// fails the assert so we exit the program immediately
		char errorstr[5000];
		sprintf(errorstr,"ASSERT FAILURE (%s) in %s on line %d (function %s).\n",assertstring,filenamestring.c_str(),linenumber,functionnamestring.c_str());
		fprintf(stderr,"%s",errorstr);
		fprintf(stdout,"%s",errorstr);
		WindowsMessage(errorstr,"ASSERT FAILURE");
		exit(1);
		}
	}
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// test support
void JrDebug::operator()(const JrdTestMsgType inmtype, const bool assertcondition, const char *assertstring, const char *assertstring2)
    {
	// test helpers
	jrd_stdstringT assertstr;
	assertstr = jrd_stdstringT(assertstring);
	if (strcmp(assertstring2,"")!=0)
	    assertstr+=jrd_stdstringT(": ")+jrd_stdstringT(assertstring2);
	
	if (assertcondition)
		{
		// ATTN: temporary kludge
		if (displaypassingasserts || true)
			operator()(JrdTestPass,"","",0,"%s",assertstr.c_str());
		}
	else
		operator()(JrdTestFail,"","",100,"%s",assertstr.c_str());

	if (inmtype==JrdTestMsgStop && !assertcondition)
		{
		// fails the assert so we exit the program immediately
		char myerrorstr[5000];
		strcpy(myerrorstr,"hello");
		sprintf(myerrorstr,"CRTICIAL TEST FAILURE (%s) in %s on line %d (function %s).\n",assertstr.c_str(),filenamestring.c_str(),linenumber,functionnamestring.c_str());
		throw JrdTestException(myerrorstr);
		//fprintf(stderr,"%s",errorstr);
		//fprintf(stdout,"%s",errorstr);
		//WindowsMessage(errorstr,"EXCEPTIONAL TEST FAILURE");
		//exit(1);
		}
	}
//---------------------------------------------------------------------------










//---------------------------------------------------------------------------
// snprintf replacements for when compiler doesnt support snprintf
//  these are less than ideal since they cannot check for buffer overflow..
int JrDebug::mysnprintf_rep(char *outstr, int maxlen, const char* format, ...)
{

	// just call builtin and hope it doesnt overflow
	va_list args;
	va_start( args, format );
	int retv=vsprintf( outstr, format, args );
	va_end( args );
	outstr[maxlen-1]='\0';
	return retv;
}

int JrDebug::myvsnprintf_rep(char *outstr, int maxlen, const char *format, va_list &va )
{


	// just call builtin and hope it doesnt overflow
	int retv=vsprintf( outstr, format, va );
	outstr[maxlen-1]='\0';
	return retv;
}
//---------------------------------------------------------------------------














































//---------------------------------------------------------------------------
// stream class methods
void JrDebugs::setvalues(const JrdType inmtype , const char *indmsubtypestr, const char *inactivity, const int inseverity)
	{
	// save extra info for subsequent call
	extrainfo=true;
	mtype=inmtype;
	dmsubtypestr=jrd_stdstringT(indmsubtypestr);
	activity=jrd_stdstringT(inactivity);
	severity=inseverity;
	}

JrDebugs::JrDebugs(const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3,JrdType inmtype)
	: jrd_stdostringstreamT()
	{
	// this class mainly just wraps a normal JrDebug class
	mtype=inmtype;
	jrd = new JrDebug(infilename,ilinenum,intimestamp,infunctionname,infunctionname2,infunctionname3,inmtype);
	// start off with no extra info
	extrainfo=false;
	}

JrDebugs::JrDebugs()
	: jrd_stdostringstreamT()
	{
	// this constructor should never be created its just for my testing
	jrd = new JrDebug("",0,"","","","",JrdMessage);
	// start off with no extra info
	extrainfo=false;
	}

JrDebugs::~JrDebugs()
	{
	// send output to our JrDebug item
	if (!extrainfo)
		{
		// this is a problem.  this means that user has done line debugout << "hello";
		// this creates a very strange conflict with the use of temporary objects
		//  see http://groups.google.com/groups?selm=qrvkjvk67b6539rpngrjc6cs20v162cup8%404ax.com
		// so we flag this as an error - BUT actually our macros should prevent this from ever occuring.
		//  im leaving it in to remind me why the macro for JrDebugInternalStream is strange
		jrd->operator() ("***** JRDEBUG INVOCATION ERROR ***** -> you cannot use 'debugout << someinfo' , you need to call debugout() << 'someinfo'");
		}
	else
		jrd->operator()(mtype,dmsubtypestr.c_str(),activity.c_str(),severity,str());

	// Clear the stringstream buffer (ATTN: is this necesary in order to flush ourselves?)
	str(jrd_stdstringT());
	// delete
	delete jrd;
	}

JrDebugs& JrDebugs::operator()()
	{
	// called with no information (we require this due to bug)
	setvalues(mtype,"","",JrDebugDefaultLevel);
	return *this;
	}
JrDebugs& JrDebugs::operator()(const JrdType inmtype , const int inseverity)
	{
	// save extra info for subsequent call
	setvalues(inmtype,"","",inseverity);
	return *this;
	}
JrDebugs& JrDebugs::operator()(const JrdType inmtype , const char *indmsubtypestr, const char *inactivity, const int inseverity)
	{
	// save extra info for subsequent call
	setvalues(inmtype,indmsubtypestr,inactivity,inseverity);
	return *this;
	}
//---------------------------------------------------------------------------

