//---------------------------------------------------------------------------
// jrdebug_output_formatters_csv.h
// derived methods for different outputs
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Csv (comma separated fields)
// register the factory
JrDebugRegisterOutputFormatter(JrDebugRegisteredOutputFormatter_Csv);
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// constructor and destructor
JrDebugRegisteredOutputFormatter_Csv::JrDebugRegisteredOutputFormatter_Csv(bool isfactory)
:JrDebugRegisteredOutputFormatter()
{
	// register if appropraite
	if (isfactory)
		{
		// create a NEW object for registering, dont try to attach this global one
		JrDebugSingletonInsurer dummy(false);
		JrDebug::RegisterFormatterFactory(MakeNew());
		return;
		}
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// shutdown and startup
bool JrDebugRegisteredOutputFormatter_Csv::Startup(const char *optionstring)
{
	// process optionstring
	if (strcmp(optionstring,"")!=0)
		return false;
	// return success
	return true;
}

bool JrDebugRegisteredOutputFormatter_Csv::Shutdown()
{
	// virtual shut down (any open files will be closed by caller when we return)
	return true;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// HandleDebugMessage should format the user's string and call DoOutput one or more times
// Note it will be invoked with any user % args already repalced
void JrDebugRegisteredOutputFormatter_Csv::HandleDebugMessage(JrDebug *jrd,const char *str)
{
	// csv style
	char strout[JrDebug_MaxBufferSize];

	// function name is present?
	jrd_stdstringT functionnamestring = jrd->get_functionnamestring();
	// file info
	jrd_stdstringT filedirstring = jrd->get_filedirstring();
	jrd_stdstringT filenamestring = jrd->get_filenamestring();
	// misc
	jrd_stdstringT messagetypestring = jrd->get_messagetypestring();
	jrd_stdstringT timestampstring = jrd->get_timestampstring();
	long linenumber=jrd->get_linenumber();
	long runid=jrd->get_runid();
	int severitylevelnumber=jrd->get_severitylevelnumber();
	long debugindex=jrd->get_debugindex();
	// get current time
	time_t timenum;
	time(&timenum);

	// construct the format string to display (uses snprintf for safety to insure we dont go over buffer size)
	mysnprintf(strout, JrDebug_MaxBufferSize-1, "%ld , %ld , %ld , \"%s\" , %d , \"%s\" , \"%s\" , \"%s\" , %ld , \"%s\" , \"%s\"\n", \
		runid, (long)timenum, debugindex, messagetypestring.c_str(),severitylevelnumber, filenamestring.c_str() , filedirstring.c_str(), timestampstring.c_str(), linenumber, functionnamestring.c_str(), str );
	strout[JrDebug_MaxBufferSize-1] = 0;
	DoOutput(strout);
}
//---------------------------------------------------------------------------
