//---------------------------------------------------------------------------
// jrdebug_output_formatters_html.h
// derived methods for different outputs
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// html (using tables)
// register the factory
JrDebugRegisterOutputFormatter(JrDebugRegisteredOutputFormatter_Html);
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// constructor and destructor
JrDebugRegisteredOutputFormatter_Html::JrDebugRegisteredOutputFormatter_Html(bool isfactory)
:JrDebugRegisteredOutputFormatter()
{
	// register if appropraite
	if (isfactory)
		{
		// create a NEW object for registering, dont try to attach this global one
		JrDebugSingletonInsurer dummy(false);
		JrDebug::RegisterFormatterFactory(MakeNew());
		return;
		}
	// init
	lastblank=false;
	wantblank=true;
	wantstartblank=false;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// shutdown and startup
bool JrDebugRegisteredOutputFormatter_Html::Startup(const char *optionstring)
{
	// process optionstring
	if (strcmp(optionstring,"")!=0)
		return false;

	// Color scheme based on cpp-unit test project

	// header
	DoOutput("<HTML>\n");
	DoOutput("<TITLE>JrDebug Output</TITLE>\n");
	// class styles
	DoOutput("<HEAD>\n");
	DoOutput("<style type=\"text/css\" media=\"screen\"><!--\n");
	DoOutput(".tc1 { border-width: 1px; border-style: dotted; }\n");
	DoOutput(".c1 { border-width: 1px; border-style: dotted; text-align:left; }\n");
	DoOutput(".cc { border-width: 1px; border-style: dotted; text-align:center; }\n");	
	DoOutput(".ccb { border-width: 1px; border-style: groove; text-align:center; }\n");	
	DoOutput(".ch { border-width: 1px; border-style: groove; text-align:center; color:#0066FF; font-weight: bold; }\n");
	DoOutput(".chL { border-width: 1px; border-style: dotted; text-align:left; }\n");
	DoOutput(".rt { border-style: dotted; background-color: #a5cef7; font-weight: bold; }\n");
	DoOutput(".rbl { border-style: dotted; }\n");
	DoOutput(".rn { border-style: dotted; background-color: #efefe7; }\n");
	DoOutput(".rb { border-style: dotted; background-color: #E5E5E0;}\n");
	DoOutput(".re { border-style: dotted; background-color: #efefe7; color:#FF0808;}\n");
	DoOutput(".rw { border-style: dotted; background-color: #efefe7; color:#F832C6;}\n");
	DoOutput(".reb { border-style: dotted; background-color: #E5E5E0; }\n");
	DoOutput("--></style>\n");
	DoOutput("</HEAD>\n");
	// body and table
	DoOutput("<BODY>\n");
	DoOutput("<TABLE border=\"0\" width=\"100%\" cellspacing=\"3\" cellpadding=\"0\" class=\"tc1\">\n");
	//
	DoOutput("<TR class=\"rt\"><TD colspan=\"6\" class=\"c1\">");
	DoOutput((GetHeaderAnnounceString()).c_str());
	DoOutput("</TD></TR>\n");
	//
	DoOutput("<TR class=\"rt\"><TD class=\"c1\">Message</TD> <TD class=\"cc\">Result</TD> <TD class=\"c1\">Type</TD> <TD class=\"c1\">FileName</TD> <TD class=\"c1\">Line</TD> <TD class=\"c1\">Function</TD></TR>\n");
	return true;
}

bool JrDebugRegisteredOutputFormatter_Html::Shutdown()
{
	// footer
	DoOutput("</TABLE>\n");
	DoOutput("</BODY>\n");
	return true;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// HandleDebugMessage should format the user's string and call DoOutput one or more times
// Note it will be invoked with any user % args already repalced
void JrDebugRegisteredOutputFormatter_Html::HandleDebugMessage(JrDebug *jrd,const char *str)
{
	// csv style
	char strout[JrDebug_MaxBufferSize];
	char colclass[10],colclassmsg[10],rowclass[10],colclassresult[10];
	//
	jrd_stdstringT strstring=jrd_stdstringT(str);
	// function name is present?
	jrd_stdstringT functionnamestring = jrd->get_functionnamestring();
	// file info
	jrd_stdstringT filedirstring = jrd->get_filedirstring();
	jrd_stdstringT filenamestring = jrd->get_filenamestring();
	// jrd_stdstringT fullfilenamestring=filedirstring + filenamestring;
	// misc
	jrd_stdstringT messagetypestring = jrd->get_messagetypestring();
	jrd_stdstringT timestampstring = jrd->get_timestampstring();
	long linenumber=jrd->get_linenumber();
	// int severitylevelnumber=jrd->get_severitylevelnumber();
	// long debugindex=jrd->get_debugindex();

	// get current time
	time_t timenum;
	time(&timenum);
	// type
	JrdType jrdtype = jrd->get_messagetype();

	// we insist on blank line before blocks start
	if (jrdtype==JrdBlockStart || wantblank || (wantstartblank && jrdtype!=JrdBlockEnd && jrdtype!=JrdBlockEndError) )
		{
		// blank line
		DoOutput("<TR class=\"rbl\"><TD colspan=\"6\" class=\"c2\">&nbsp;</TD></TR>\n");
		}
	//	
	wantstartblank=false;

	// depth, for nice indenting
	int ourdepth=jrd->get_depth();
	char indentstr[48]; strcpy(indentstr,"&nbsp;&nbsp;&nbsp;");

	// row color
	if (jrdtype==JrdBlockStart || jrdtype==JrdBlockEnd || jrdtype==JrdBlockEndError)
		{
		strcpy(rowclass,"rb");
		}
	else
		{
		strcpy(rowclass,"rn");
		}

	// error rows
	if (jrd->MessageIsSimpleError())
		strcpy(rowclass,"re");

	// col
	strcpy(colclass,"c1");
	strcpy(colclassmsg,"c1");
	strcpy(colclassresult,"cc");

	// ATTN:
	// we would like to strip off the (## sec., #/# err.) and move it to its own column
	// and color red this cell if errors > 0
	jrd_stdstringT resultnumstr;
	if (jrdtype==JrdBlockEnd || jrdtype==JrdBlockEndError )
		{
		//strcpy(colclassresult,"ccb");
		// modify strstring to remove details after the (
		int len=(int)(strlen(str));
		int count;
		for (count=len-1;count>0;--count)
			{
			if (str[count]=='(')
				break;
			}
		if (count>0)
			{
			resultnumstr=jrd_stdstringT(&str[count+1],len-(count+3));
			strstring=jrd_stdstringT(str,count-1);
			}
		else
			resultnumstr="&nbsp;";
		//
		if (strstr(&str[count+1],"err")!=NULL)
			{
			strcpy(rowclass,"reb");
			strcpy(colclassmsg,"chL");
			strcpy(colclassresult,"ch");
			}
		}
	else
		resultnumstr="&nbsp;";

	// prefix and indent
	mysnprintf(strout, JrDebug_MaxBufferSize-1, "<TR class=\"%s\"><TD class=\"%s\">",rowclass,colclassmsg); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);

	if ( jrd->get_Filter_OnlyErrors() != true || jrd->get_Filter_ShowAllBlocks()==true)
		DoOutput(indentstr,ourdepth);

	// construct the format string to display (uses snprintf for safety to insure we dont go over buffer size)
	mysnprintf(strout, JrDebug_MaxBufferSize-1, "%s</TD> <TD class=\"%s\">%s</TD> ", strstring.c_str(), colclassresult,resultnumstr.c_str());
	strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	//mysnprintf(strout, JrDebug_MaxBufferSize-1, "<TD class=\"%s\">%s</TD> <TD class=\"%s\"><A href=\"%s\">%s</A></TD> <TD class=\"%s\">%ld</TD> <TD class=\"%s\">%s</TD></TR>\n",colclass, messagetypestring.c_str(), colclass,fullfilenamestring.c_str(),filenamestring.c_str() , colclass,linenumber, colclass, functionnamestring.c_str() );
	mysnprintf(strout, JrDebug_MaxBufferSize-1, "<TD class=\"%s\">%s</TD> <TD class=\"%s\">%s</TD> <TD class=\"%s\">%ld</TD> <TD class=\"%s\">%s</TD></TR>\n",colclass, messagetypestring.c_str(), colclass,filenamestring.c_str() , colclass,linenumber, colclass, functionnamestring.c_str() );
	strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);

	// we insist on blank line after blocks end
	if (jrdtype==JrdBlockEnd || jrdtype==JrdBlockEndError)
		{
		if (JrDebug::get_Filter_OnlyErrors() && JrDebug::get_Filter_ShowAllBlocks())
			{
			wantstartblank=true;
			wantblank=false;
			}
		else
			wantblank=true;
		}
	else
		wantblank=false;
}
//---------------------------------------------------------------------------
